@@ -844,6 +844,7 @@ public:
     void add_attribute(const std::string& key, const int64_t& value, int64_t default_value=0);
     void add_attribute(const std::string& key, const bool& value, bool default_value=false);
     void add_attribute(const std::string& key, const float& value, float default_value=0.f);
+    void add_attribute(const std::string& key, const double& value, double default_value=0.f);
     void add_attribute(const std::string& key, const std::string& value, const std::string& default_value);
 
     template <class T>
@@ -927,8 +928,11 @@ public:
         SerializerXml child = key.empty() ? *this : add_child(key);
         for (const intrusive_ptr<T>& value : values)
         {
-            SerializerXml item = child.add_child(value->get_type());
-            value->serialize_xml(item);
+            SerializerXml item = child.add_child(value ? value->get_type() : "");
+            if(value)
+            {
+                value->serialize_xml(item);
+            }
         }
     }
 
@@ -1000,13 +1004,9 @@ public:
         SerializerXml child = key.empty() ? *this : add_child(key);
         for (auto& pair : values)
         {
-            if(!pair.second)
-            {
-                continue;
-            }
             SerializerXml item = child.add_child("pair");
             item.add_attribute("key", pair.first, default_value::value<Key>());
-            item.add_attribute("value", pair.second->name, default_value::value<std::string>());            
+            item.add_attribute("value", pair.second ? pair.second->name : "", default_value::value<std::string>());            
         }
     }
 
@@ -1064,13 +1064,9 @@ public:
         SerializerXml child = key.empty() ? *this : add_child(key);
         for (auto& pair : values)
         {
-            if(!pair.second)
-            {
-                continue;
-            }
             SerializerXml item = child.add_child("pair");
             item.add_attribute("key", pair.first.str(), default_value::value<std::string>());
-            item.add_attribute("value", pair.second->name, default_value::value<std::string>());            
+            item.add_attribute("value", pair.second ? pair.second->name : "", default_value::value<std::string>());            
         }
     }
 
@@ -1098,12 +1094,8 @@ public:
         SerializerXml child = key.empty() ? *this : add_child(key);
         for (auto& pair : values)
         {
-            if(!pair.first)
-            {
-                continue;
-            }
             SerializerXml item = child.add_child("pair");
-            item.add_attribute("key", pair.first->name, default_value::value<std::string>());
+            item.add_attribute("key", pair.first ? pair.first->name : "", default_value::value<std::string>());
             item.add_attribute("value", pair.second, default_value::value<Value>());            
         }
     }
@@ -1117,12 +1109,8 @@ public:
         SerializerXml child = key.empty() ? *this : add_child(key);
         for (auto& pair : values)
         {
-            if(!pair.first)
-            {
-                continue;
-            }
             SerializerXml item = child.add_child("pair");
-            item.add_attribute("key", pair.first->name, default_value::value<std::string>());
+            item.add_attribute("key", pair.first ? pair.first->name : "", default_value::value<std::string>());
             item.add_attribute("value", pair.second.str(), default_value::value<std::string>());            
         }
     }
@@ -1136,13 +1124,9 @@ public:
         SerializerXml child = key.empty() ? *this : add_child(key);
         for (auto& pair : values)
         {
-            if(!pair.first || !pair.second)
-            {
-                continue;
-            }
             SerializerXml item = child.add_child("pair");
-            item.add_attribute("key", pair.first->name, default_value::value<std::string>());
-            item.add_attribute("value", pair.second->name, default_value::value<std::string>());            
+            item.add_attribute("key", pair.first ? pair.first->name : "", default_value::value<std::string>());
+            item.add_attribute("value", pair.second ? pair.second->name : "", default_value::value<std::string>());            
         }
     }
 
@@ -1155,12 +1139,8 @@ public:
         SerializerXml child = key.empty() ? *this : add_child(key);
         for (auto& pair : values)
         {
-            if(!pair.first)
-            {
-                continue;
-            }
             SerializerXml item = child.add_child("pair");
-            item.add_attribute("key", pair.first->name, default_value::value<std::string>());
+            item.add_attribute("key", pair.first ? pair.first->name : "", default_value::value<std::string>());
             item.serialize(pair.second, "value");            
         }
     }
@@ -1204,13 +1184,9 @@ public:
         SerializerXml child = key.empty() ? *this : add_child(key);
         for (auto& pair : values)
         {
-            if(!pair.second)
-            {
-                continue;
-            }
             SerializerXml item = child.add_child("pair");
             item.serialize(pair.first, "key");
-            item.add_attribute("value", pair.second->name, default_value::value<std::string>());
+            item.add_attribute("value", pair.second ? pair.second->name : "", default_value::value<std::string>());
         }
     }
 
@@ -1251,6 +1227,7 @@ public:
     int64_t get_attribute(const std::string& key, int64_t default_value=0);
     bool get_attribute(const std::string& key, bool default_value=false);
     float get_attribute(const std::string& key, float default_value=0.f);
+    double get_attribute(const std::string& key, double default_value=0.f);
     std::string get_attribute(const std::string& key, const std::string& default_value);
 
     DeserializerXml begin();
@@ -1332,8 +1309,8 @@ public:
             if(object)
             {
                 object->deserialize_xml(item);
-                values.push_back(object);
             }
+            values.push_back(object);
         }
     }
 
@@ -1648,6 +1625,14 @@ void SerializerXml::add_attribute(const std::string &key, const float &value, fl
     }
 }
 
+void SerializerXml::add_attribute(const std::string &key, const double &value, double default_value)
+{
+    if (value != default_value)
+    {
+        _node->append_attribute(key.c_str()).set_value(value);
+    }
+}
+
 void SerializerXml::add_attribute(const std::string &key, const std::string &value, const std::string &default_value)
 {
     if (value != default_value)
@@ -1710,6 +1695,11 @@ float DeserializerXml::get_attribute(const std::string &key, float default_value
     return _node->attribute(key.c_str()).as_float(default_value);
 }
 
+double DeserializerXml::get_attribute(const std::string &key, double default_value)
+{
+    return _node->attribute(key.c_str()).as_double(default_value);
+}
+
 std::string DeserializerXml::get_attribute(const std::string &key, const std::string &default_value)
 {
     return _node->attribute(key.c_str()).as_string(default_value.c_str());
@@ -1781,6 +1771,7 @@ public:
     void add_attribute(const std::string &key, const int64_t &value, int64_t default_value = 0);
     void add_attribute(const std::string &key, const bool &value, bool default_value = false);
     void add_attribute(const std::string &key, const float &value, float default_value = 0.f);
+    void add_attribute(const std::string &key, const double &value, double default_value = 0.f);
     void add_attribute(const std::string &key, const std::string &value, const std::string &default_value);
 
     void add_array_item(const int &value);
@@ -1873,7 +1864,7 @@ public:
         SerializerJson child = key.empty() ? *this : add_array(key);
         for (const T &value : values)
         {
-            child.add_array_item(value->name);
+            child.add_array_item(value ? value->name : "");
         }
     }
 
@@ -1933,7 +1924,7 @@ public:
         {
             SerializerJson item = child.add_array_item();
             item.add_attribute("key", pair.first, default_value::value<Key>());
-            item.add_attribute("value", pair.second->name, default_value::value<std::string>());            
+            item.add_attribute("key", pair.second ? pair.second->name : "", default_value::value<std::string>());            
         }
     }
 
@@ -1993,7 +1984,7 @@ public:
         {
             SerializerJson item = child.add_array_item();
             item.add_attribute("key", pair.first.str(), default_value::value<std::string>());
-            item.add_attribute("value", pair.second->name, default_value::value<std::string>());            
+            item.add_attribute("key", pair.second ? pair.second->name : "", default_value::value<std::string>());            
         }
     }
 
@@ -2022,7 +2013,7 @@ public:
         for (auto& pair : values)
         {
             SerializerJson item = child.add_array_item();
-            item.add_attribute("key", pair.first->name, default_value::value<std::string>());
+            item.add_attribute("key", pair.first ? pair.first->name : "", default_value::value<std::string>());
             item.add_attribute("value", pair.second, default_value::value<Value>());            
         }
     }
@@ -2037,7 +2028,7 @@ public:
         for (auto& pair : values)
         {
             SerializerJson item = child.add_array_item();
-            item.add_attribute("key", pair.first->name, default_value::value<std::string>());
+            item.add_attribute("key", pair.first ? pair.first->name : "", default_value::value<std::string>());
             item.add_attribute("value", pair.second.str(), default_value::value<std::string>());            
         }
     }
@@ -2052,8 +2043,8 @@ public:
         for (auto& pair : values)
         {
             SerializerJson item = child.add_array_item();
-            item.add_attribute("key", pair.first->name, default_value::value<std::string>());
-            item.add_attribute("value", pair.second->name, default_value::value<std::string>());            
+            item.add_attribute("key", pair.first ? pair.first->name : "", default_value::value<std::string>());
+            item.add_attribute("key", pair.second ? pair.second->name : "", default_value::value<std::string>());
         }
     }
 
@@ -2067,7 +2058,7 @@ public:
         for (auto& pair : values)
         {
             SerializerJson item = child.add_array_item();
-            item.add_attribute("key", pair.first->name, default_value::value<std::string>());
+            item.add_attribute("key", pair.first ? pair.first->name : "", default_value::value<std::string>());
             item.serialize(pair.second, "value");            
         }
     }
@@ -2113,7 +2104,7 @@ public:
         {
             SerializerJson item = child.add_array_item();
             item.serialize(pair.first, "key");
-            item.add_attribute("value", pair.second->name, default_value::value<std::string>());            
+            item.add_attribute("value", pair.second ? pair.second->name : "", default_value::value<std::string>());            
         }
     }
 
@@ -2162,6 +2153,7 @@ public:
     int64_t get_attribute(const std::string &key, int64_t default_value = 0);
     bool get_attribute(const std::string &key, bool default_value = false);
     float get_attribute(const std::string &key, float default_value = 0.f);
+    double get_attribute(const std::string &key, double default_value = 0.f);
     std::string get_attribute(const std::string &key, const std::string &default_value);
 
     void get_array_item(int &value);
@@ -2552,6 +2544,14 @@ void SerializerJson::add_attribute(const std::string &key, const float &value, f
     }
 }
 
+void SerializerJson::add_attribute(const std::string &key, const double &value, double default_value)
+{
+    if (value != default_value)
+    {
+        _json[key] = value;
+    }
+}
+
 void SerializerJson::add_attribute(const std::string &key, const std::string &value, const std::string &default_value)
 {
     if (value != default_value)
@@ -2625,6 +2625,11 @@ float DeserializerJson::get_attribute(const std::string &key, float default_valu
     return _json.isMember(key) ? _json[key].asFloat() : default_value;
 }
 
+double DeserializerJson::get_attribute(const std::string &key, double default_value)
+{
+    return _json.isMember(key) ? _json[key].asFloat() : default_value;
+}
+
 std::string DeserializerJson::get_attribute(const std::string &key, const std::string &default_value)
 {
     return _json.isMember(key) ? _json[key].asString() : default_value;
@@ -2710,6 +2715,7 @@ struct is_attribute
                                    std::is_same<uint64_t, T>::value ||
                                    std::is_same<bool, T>::value ||
                                    std::is_same<float, T>::value ||
+                                   std::is_same<double, T>::value ||
                                    std::is_same<std::string, T>::value) &&
                                   !std::is_base_of<BaseEnum, T>::value;
 
@@ -2808,6 +2814,10 @@ struct default_value
     static typename std::enable_if<std::is_same<float, T>::value, float>::type
     value() { return 0.f; }
 
+    template<class T>
+    static typename std::enable_if<std::is_same<double, T>::value, double>::type
+    value() { return 0.0; }
+
     template<class T>
     static typename std::enable_if<std::is_same<std::string, T>::value, std::string>::type
     value() { return std::string(); }
